# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import datetime

from django.utils import timezone
from django.test import RequestFactory, TestCase
from django.db import models
from django.contrib.admin import ModelAdmin, site
from django.contrib.admin.views.main import ChangeList
from django.contrib.auth.models import User
from django.utils.encoding import force_text

from .filters import DateRangeFilter, PastDateRangeFilter, FutureDateRangeFilter


class MyModel(models.Model):
    created_at = models.DateTimeField()
    publish_at = models.DateTimeField()
    review_at = models.DateTimeField()

    class Meta:
        ordering = ['created_at']


class MyModelAdmin(ModelAdmin):
    list_filter = [
        ('created_at', PastDateRangeFilter),
        ('publish_at', FutureDateRangeFilter),
        ('review_at', DateRangeFilter)
    ]
    ordering = ['-id']


def select_by(dictlist):
    return [x for x in dictlist][0]


class DateRangeFilterTestCase(TestCase):
    def setUp(self):
        self.today = datetime.date.today()
        self.tomorrow = self.today + datetime.timedelta(days=1)
        self.one_week_ago = self.today - datetime.timedelta(days=7)

        self.object_one = MyModel.objects.create(
            created_at=timezone.now(),
            publish_at=timezone.now(),
            review_at=timezone.now()
        )
        self.object_two = MyModel.objects.create(
            created_at=timezone.now() - datetime.timedelta(days=7),
            publish_at=timezone.now(),
            review_at=timezone.now()
        )

        self.user = User.objects.create_user(username='test', password='top_secret')


    def get_changelist(self, request, model, modeladmin):
        if getattr(modeladmin, 'get_changelist_instance', None):
            return modeladmin.get_changelist_instance(request)

        return ChangeList(
            request, model, modeladmin.list_display,
            modeladmin.list_display_links, modeladmin.list_filter,
            modeladmin.date_hierarchy, modeladmin.search_fields,
            modeladmin.list_select_related, modeladmin.list_per_page,
            modeladmin.list_max_show_all, modeladmin.list_editable, modeladmin,
        )


    def test_get_template(self):
        self.request_factory = RequestFactory()
        modeladmin = MyModelAdmin(MyModel, site)

        request = self.request_factory.get('/')
        request.user = self.user

        changelist = self.get_changelist(request, MyModel, modeladmin)
        filterspec = changelist.get_filters(request)[0][0]
        self.assertEqual(filterspec.template, 'daterangefilter/daterangefilter.html')


    def test_datefilter(self):
        self.request_factory = RequestFactory()
        modeladmin = MyModelAdmin(MyModel, site)

        request = self.request_factory.get('/')
        request.user = self.user

        changelist = self.get_changelist(request, MyModel, modeladmin)

        queryset = changelist.get_queryset(request)

        self.assertEqual(list(queryset), [self.object_two, self.object_one])
        filterspec = changelist.get_filters(request)[0][0]
        self.assertEqual(force_text(filterspec.title), 'created at')


    def test_datefilter_filtered(self):
        self.request_factory = RequestFactory()
        modeladmin = MyModelAdmin(MyModel, site)

        request = self.request_factory.get('/', {'created_at__gte': self.today,
                                                 'created_at__lte': self.tomorrow})
        request.user = self.user

        changelist = self.get_changelist(request, MyModel, modeladmin)

        queryset = changelist.get_queryset(request)

        self.assertEqual(list(queryset), [self.object_one])
        filterspec = changelist.get_filters(request)[0][0]
        self.assertEqual(force_text(filterspec.title), 'created at')

        choice = select_by(filterspec.choices(changelist))
        self.assertEqual(choice['query_string'], '?')
        self.assertEqual(choice['field_name'], 'created_at')


    def test_datefilter_empty_values(self):
        self.request_factory = RequestFactory()
        modeladmin = MyModelAdmin(MyModel, site)

        request = self.request_factory.get('/', {'created_at__gte': '',
                                                 'created_at__lte': ''})
        request.user = self.user

        changelist = self.get_changelist(request, MyModel, modeladmin)

        queryset = changelist.get_queryset(request)

        self.assertEqual(list(queryset), [self.object_two, self.object_one])
        filterspec = changelist.get_filters(request)[0][0]
        self.assertEqual(force_text(filterspec.title), 'created at')

        choice = select_by(filterspec.choices(changelist))
        self.assertEqual(choice['query_string'], '?')
        self.assertEqual(choice['field_name'], 'created_at')



import datetime

from django.db import models
from django.contrib import admin, messages
from django.utils import timezone
from django.utils.translation import gettext_lazy as _
from django.conf import settings


class DateRangeFilter(admin.FieldListFilter):
    def __init__(self, field, request, params, model, model_admin, field_path):
        self.field_name = field_path
        self.lookup_kwarg_gte = '{}__gte'.format(field_path)
        self.lookup_kwarg_lte = '{}__lte'.format(field_path)
        self.lookup_gte = params.get(self.lookup_kwarg_gte)
        self.lookup_lte = params.get(self.lookup_kwarg_lte)
        # todo: check if this is required in default admin
        if self.lookup_gte == '':
            params.pop(self.lookup_kwarg_gte)
        if self.lookup_lte == '':
            params.pop(self.lookup_kwarg_lte)
        if self.lookup_gte and self.lookup_lte:
            self.lookup_val = '{} - {}'.format(self.lookup_gte, self.lookup_lte)
            # if we are filtering DateTimeField we should add one day to final date
            new_field_path = field_path
            nested_model = model
            while len(new_field_path.split('__')) > 1:
                related_model, new_field_path = new_field_path.split("__")[0], '__'.join(new_field_path.split("__")[1:])
                nested_model = nested_model._meta.get_field(related_model).related_model

            field = nested_model._meta.get_field(new_field_path)

            if isinstance(field, models.DateTimeField):
                try:
                    gte_date = datetime.datetime.strptime(self.lookup_gte, '%Y-%m-%d')
                    lte_date = datetime.datetime.strptime(self.lookup_lte, '%Y-%m-%d')
                    lte_date = lte_date + datetime.timedelta(seconds=3600*24-1)
                    if settings.USE_TZ:
                        gte_date = timezone.make_aware(gte_date, timezone.get_current_timezone())
                        lte_date = timezone.make_aware(lte_date, timezone.get_current_timezone())
                    params[self.lookup_kwarg_gte] = gte_date.strftime('%Y-%m-%d %H:%M:%S%z')
                    params[self.lookup_kwarg_lte] = lte_date.strftime('%Y-%m-%d %H:%M:%S%z')
                except ValueError:
                    messages.add_message(request, messages.ERROR, _("Invalid date for '%(field_name)s' field range filter") % {'field_name': field.verbose_name})
        else:
            self.lookup_val = ''
        super(DateRangeFilter, self).__init__(field, request, params, model, model_admin, field_path)

    def get_template(self):
        if 'suit' in settings.INSTALLED_APPS:
            return 'daterangefilter/suit_daterangefilter.html'
        else:
            return 'daterangefilter/daterangefilter.html'

    def choices(self, changelist):
        yield {
            'field_name': self.field_path,
            'ranges_template': self.ranges_template,
            'value': self.lookup_val,
            'query_string': changelist.get_query_string(remove=self._get_expected_fields())
        }

    def expected_parameters(self):
        return self._get_expected_fields()

    def _get_expected_fields(self):
        return [self.lookup_kwarg_gte, self.lookup_kwarg_lte]

    template = property(get_template)
    ranges_template = None


class FutureDateRangeFilter(DateRangeFilter):
    ranges_template = 'daterangefilter/_future_ranges.html'


class PastDateRangeFilter(DateRangeFilter):
    ranges_template = 'daterangefilter/_past_ranges.html'



import setuptools

with open('README.md', 'r') as fh:
    long_description = fh.read()

setuptools.setup(
    name='django-daterangefilter',
    version='1.0.1',
    license='MIT',
    author='Andrey Novikov',
    author_email='novikov@gmail.com',
    description='Date range filter for Django admin',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/andreynovikov/django-daterangefilter/tree/master',
    project_urls={
        'Source': 'https://github.com/andreynovikov/django-daterangefilter/',
        'Tracker': 'https://github.com/andreynovikov/django-daterangefilter/issues'
    },
    packages=setuptools.find_packages(),
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Framework :: Django',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
)
